changeset in /hg/icedtea: 2008-03-10 Gary Benson <gbenson at redh...

Gary Benson gbenson at redhat.com
Thu May 29 14:12:25 PDT 2008


changeset 6ca632b6ab09 in /hg/icedtea
details: http://icedtea.classpath.org/hg/icedtea?cmd=changeset;node=6ca632b6ab09
description:
	2008-03-10  Gary Benson  <gbenson at redhat.com>

		* ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp
		(current_stack_region): Rewritten to cope with guard pages and ia64.
		(default_stack_size, default_guard_size): Comment fixes.

diffstat:

2 files changed, 74 insertions(+), 64 deletions(-)
ChangeLog                                                |    6 
ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp |  132 +++++++-------

diffs (166 lines):

diff -r 30bb0b3e0d60 -r 6ca632b6ab09 ChangeLog
--- a/ChangeLog	Fri Mar 07 12:56:28 2008 -0500
+++ b/ChangeLog	Mon Mar 10 10:37:21 2008 -0400
@@ -1,3 +1,9 @@ 2008-03-07  Joshua Sumali  <jsumali at redh
+2008-03-10  Gary Benson  <gbenson at redhat.com>
+
+	* ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp
+	(current_stack_region): Rewritten to cope with guard pages and ia64.
+	(default_stack_size, default_guard_size): Comment fixes.
+
 2008-03-07  Joshua Sumali  <jsumali at redhat.com>
 
 	* rt/net/sourceforge/jnlp/security/CertsInfoPane.java: Fix null	pointer.
diff -r 30bb0b3e0d60 -r 6ca632b6ab09 ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp
--- a/ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp	Fri Mar 07 12:56:28 2008 -0500
+++ b/ports/hotspot/src/os_cpu/linux_zero/vm/os_linux_zero.cpp	Mon Mar 10 10:37:21 2008 -0400
@@ -171,7 +171,6 @@ bool os::Linux::supports_variable_stack_
 
 size_t os::Linux::default_stack_size(os::ThreadType thr_type)
 {
-  // default stack size (compiler thread needs larger stack)
 #ifdef _LP64
   size_t s = (thr_type == os::compiler_thread ? 4 * M : 1 * M);
 #else
@@ -182,73 +181,78 @@ size_t os::Linux::default_stack_size(os:
 
 size_t os::Linux::default_guard_size(os::ThreadType thr_type)
 {
   // Creating guard page is very expensive. Java thread has HotSpot
   // guard page, only enable glibc guard page for non-Java threads.
   return (thr_type == java_thread ? 0 : page_size());
 }
 
-// Java thread:
-//
-//   Low memory addresses
-//    +------------------------+
-//    |                        |\  JavaThread created by VM does not have glibc
-//    |    glibc guard page    | - guard, attached Java thread usually has
-//    |                        |/  1 page glibc guard.
-// P1 +------------------------+ Thread::stack_base() - Thread::stack_size()
-//    |                        |\
-//    |  HotSpot Guard Pages   | - red and yellow pages
-//    |                        |/
-//    +------------------------+ JavaThread::stack_yellow_zone_base()
-//    |                        |\
-//    |      Normal Stack      | -
-//    |                        |/
-// P2 +------------------------+ Thread::stack_base()
-//
-// Non-Java thread:
-//
-//   Low memory addresses
-//    +------------------------+
-//    |                        |\
-//    |  glibc guard page      | - usually 1 page
-//    |                        |/
-// P1 +------------------------+ Thread::stack_base() - Thread::stack_size()
-//    |                        |\
-//    |      Normal Stack      | -
-//    |                        |/
-// P2 +------------------------+ Thread::stack_base()
-//
-// ** P1 (aka bottom) and size ( P2 = P1 - size) are the address and stack size returned from
-//    pthread_attr_getstack()
-
-static void current_stack_region(address* bottom, size_t* size) {
+static void current_stack_region(address *bottom, size_t *size)
+{
+  pthread_attr_t attr;
+  int res = pthread_getattr_np(pthread_self(), &attr);
+  if (res != 0) {
+    if (res == ENOMEM) {
+      vm_exit_out_of_memory(0, "pthread_getattr_np");
+    }
+    else {
+      fatal1("pthread_getattr_np failed with errno = %d", res);
+    }
+  }
+
+  address stack_bottom;
+  size_t stack_bytes;
+  res = pthread_attr_getstack(&attr, (void **) &stack_bottom, &stack_bytes);
+  if (res != 0) {
+    fatal1("pthread_attr_getstack failed with errno = %d", res);
+  }
+  address stack_top = stack_bottom + stack_bytes;
+
+  // The block of memory returned by pthread_attr_getstack() includes
+  // guard pages where present.  We need to trim these off.
+  size_t page_bytes = os::Linux::page_size();
+  assert(((intptr_t) stack_bottom & (page_bytes - 1)) == 0, "unaligned stack");
+
+  size_t guard_bytes;
+  res = pthread_attr_getguardsize(&attr, &guard_bytes);
+  if (res != 0) {
+    fatal1("pthread_attr_getguardsize failed with errno = %d", res);
+  }
+  int guard_pages = align_size_up(guard_bytes, page_bytes) / page_bytes;
+  assert(guard_bytes == guard_pages * page_bytes, "unaligned guard");
+
+#ifdef IA64
+  // IA64 has two stacks sharing the same area of memory, a normal
+  // stack growing downwards and a register stack growing upwards.
+  // Guard pages, if present, are in the centre.  This code splits
+  // the stack in two even without guard pages, though in theory
+  // there's nothing to stop us allocating more to the normal stack
+  // or more to the register stack if one or the other were found
+  // to grow faster.
+  int total_pages = align_size_down(stack_bytes, page_bytes) / page_bytes;
+  stack_bottom += (total_pages - guard_pages) / 2 * page_bytes;
+#endif // IA64
+
+  stack_bottom += guard_bytes;
+
+  pthread_attr_destroy(&attr);
+
+  // The initial thread has a growable stack, and the size reported
+  // by pthread_attr_getstack is the maximum size it could possibly
+  // be given what currently mapped.  This can be huge, so we cap it.
   if (os::Linux::is_initial_thread()) {
-     // initial thread needs special handling because pthread_getattr_np()
-     // may return bogus value.
-     *bottom = os::Linux::initial_thread_stack_bottom();
-     *size = os::Linux::initial_thread_stack_size();
-  } else {
-     pthread_attr_t attr;
-
-     int rslt = pthread_getattr_np(pthread_self(), &attr);
-
-     // JVM needs to know exact stack location, abort if it fails
-     if (rslt != 0) {
-       if (rslt == ENOMEM) {
-         vm_exit_out_of_memory(0, "pthread_getattr_np");
-       } else {
-         fatal1("pthread_getattr_np failed with errno = %d", rslt);
-       }
-     }
-
-     if (pthread_attr_getstack(&attr, (void **)bottom, size) != 0 ) {
-         fatal("Can not locate current stack attributes!");
-     }
-
-     pthread_attr_destroy(&attr);
-
-  }
-  assert(os::current_stack_pointer() >= *bottom &&
-         os::current_stack_pointer() < *bottom + *size, "just checking");
+    stack_bytes = stack_top - stack_bottom;
+
+    if (stack_bytes > JavaThread::stack_size_at_create())
+      stack_bytes = JavaThread::stack_size_at_create();
+
+    stack_bottom = stack_top - stack_bytes;
+  }
+
+  assert(os::current_stack_pointer() >= stack_bottom, "should do");
+  assert(os::current_stack_pointer() < stack_top, "should do");
+
+  *bottom = stack_bottom;
+  *size = stack_top - stack_bottom;
 }
 
 address os::current_stack_base()
@@ -182,73 +181,78 @@ size_t os::Linux::default_stack_size(os:
 size_t os::Linux::default_guard_size(os::ThreadType thr_type)
 {
-  // Creating guard page is very expensive. Java thread has HotSpot
-  // guard page, only enable glibc guard page for non-Java threads.
+  // Only enable glibc guard pages for non-Java threads
+  // (Java threads have HotSpot guard pages)
   return (thr_type == java_thread ? 0 : page_size());
 }



More information about the distro-pkg-dev mailing list